package cn.estsh.i3plus.ext.mes.pcn.api.busi;

import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesContainerPackageDetailContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerPackage;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerPackageDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerSnStation;
import cn.estsh.i3plus.pojo.mes.bean.MesStation;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

public interface IMesStationContainerSnExtService {

    @ApiOperation(value = "根据设备代码查询站点信息")
    List<MesStation> getStationList(String organizeCode, String equipmentCode);

    @ApiOperation(value = "获取站点关联的容器条码")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    Map<String, List<MesContainerSnStation>> getContainerSnStationMap(String organizeCode, List<MesStation> stationList);

    @ApiOperation(value = "获取站点关联的容器条码")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    Map<String, List<MesContainerSnStation>> getContainerSnStationTopMap(String organizeCode, List<MesStation> stationList);

    @ApiOperation(value = "获取站点关联的容器条码")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerSnStation> getContainerSnStationList(String organizeCode, List<MesStation> stationList);

    @ApiOperation(value = "获取站点关联的容器条码")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerSnStation> getContainerSnStationListByStation(String organizeCode, List<String> stationList);

    @ApiOperation(value = "根据容器条码查询容器条码与站点的对应关系")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    MesContainerSnStation getContainerSnStation(String organizeCode, String containerSn);

    @ApiOperation(value = "根据站点查询容器条码与站点的对应关系")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    MesContainerSnStation getContainerSnStationByStation(String organizeCode, String station);

    @ApiOperation(value = "获取容器条码上料明细表信息【可扣减】【根据 主表seq,主表createDatetime,明细表createDatetime 正序】")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerPackageDetailContext> getContainerPackageDetailContext(String organizeCode, List<MesStation> stationList);

    @ApiOperation(value = "获取容器条码上料主表信息")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerPackage> getContainerPackageList(String organizeCode, List<MesContainerSnStation> containerSnStationList, Integer packageStatus);

    @ApiOperation(value = "获取容器条码上料主表信息")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerPackage> getContainerPackageListByContainerSn(String organizeCode, List<String> containerSnList, Integer packageStatus);

    @ApiOperation(value = "获取容器条码上料主表信息【进料】的一条数据")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    MesContainerPackage getContainerPackageByContainerSn(String organizeCode, List<String> containerSnList, Integer packageStatus);

    @ApiOperation(value = "根据ID查询上料主表信息")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    MesContainerPackage getContainerPackageById(String organizeCode, Long id);

    @ApiOperation(value = "获取容器条码上料明细表信息【可扣减】")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerPackageDetail> getContainerPackageDetailList(String organizeCode, List<MesContainerPackage> containerPackageList);

    @ApiOperation(value = "获取容器条码上料明细表信息【可扣减】")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerPackageDetail> getContainerPackageDetailListByPid(String organizeCode, List<Long> pidList);

    @ApiOperation(value = "获取容器条码上料明细表信息")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerPackageDetail> getContainerPackageDetailListByPid(String organizeCode, List<Long> pidList, Integer snStatus);

    @ApiOperation(value = "获取容器条码上料明细表信息")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    List<MesContainerPackageDetail> getContainerPackageDetailListByPid(String organizeCode, Long pid);

    @ApiOperation(value = "根据单件条码查询上料明细表信息")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    MesContainerPackageDetail getContainerPackageDetailByBarCode(String organizeCode, String barCode);

    @ApiOperation(value = "获取容器条码的一条上料明细表信息【可扣减】")
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    Boolean checkIsExistContainerPackageDetail(String organizeCode, Long id);

    @ApiOperation(value = "写入扣料记录")
    void insertContainerPackageDetailFeeding(StationRequestBean reqBean, MesContainerPackageDetailContext containerPackageDetailContext, MesProductionPsOutContext productionPsOutContext);

    @ApiOperation(value = "修改上料明细表信息状态")
    void updateContainerPackageDetail(String organizeCode, String userInfo, Integer snStatus, Long... ids);

    @ApiOperation(value = "写入容器条码与站点的关系信息")
    void insertContainerSnStationList(String organizeCode, String userInfo, String station, List<MesContainerPackage> containerPackageList, String topContainerSn);

}
