package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesRawSnWeightRecordService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdRuleContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyNosortContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.MesRawMixCfgDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesRawSnWeightRecordLogRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesRawSnWeightRecordRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesRawWeightCfgRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.util.List;
import java.util.Objects;

@Service
public class MesRawSnWeightRecordServiceImpl implements IMesRawSnWeightRecordService {
    @Autowired
    private MesRawSnWeightRecordRepository rawSnWeightRecordRDao;
    @Autowired
    private MesRawWeightCfgRepository rawWeightCfgRDao;
    @Autowired
    private MesRawMixCfgDetailRepository rawMixCfgDetailRDao;
    @Autowired
    private IMesPartService partService;
    @Autowired
    private MesRawSnWeightRecordLogRepository rawSnWeightRecordLogRDao;

    @Override
    public void insertRawRemainWeightRecord(MesProductionPsOutContext productionPsOutContext, MesProductionAssemblyContext context, String organizeCode, String username) {
        MesRawMixCfgDetail detail = rawMixCfgDetailRDao.getByProperty(
                new String[]{"organizeCode", "isValid", "isDeleted", "outPartNo", "partNo"},
                new Object[]{organizeCode, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue(), context.getPartNo(), context.getAssemblyPartNo()});
        if (detail == null) {
            return;
        }

        MesRawWeightCfg weightCfg = rawWeightCfgRDao.getByProperty(new String[]{"organizeCode", "isValid", "isDeleted", "partNo"},
                new Object[]{organizeCode, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue(), context.getAssemblyPartNo()});
        if (weightCfg == null) {
            return;
        }
        double remainQty = getRemainQty(organizeCode, context.getAssemblyPartNo(), context.getAssemblySn());
        double weightQty = context.getQty() != null ? context.getQty() : 0;
        // 保留两位小数，四舍五入
        MathContext mc = new MathContext(8, RoundingMode.HALF_UP);
        BigDecimal decimalRemainQty = new BigDecimal(remainQty, mc);
        BigDecimal decimalWeight = new BigDecimal(weightQty, mc);

        MesRawSnWeightRecord record = new MesRawSnWeightRecord();
        record.setOrganizeCode(organizeCode);
        record.setBarCode(context.getAssemblySn());
        record.setPartNo(context.getAssemblyPartNo());
        record.setPartName(context.getAssemblyPartName());
//        record.setNetWeight(remainQty - context.getQty());
        record.setNetWeight(decimalRemainQty.subtract(decimalWeight).doubleValue());
        record.setNetUnit(weightCfg.getNetUnit());
        record.setRawType(detail.getRawType());
        ConvertBean.serviceModelInitialize(record, username);
        rawSnWeightRecordRDao.insert(record);

        MesRawSnWeightRecordLog recordLog = new MesRawSnWeightRecordLog();
        recordLog.setOrganizeCode(organizeCode);
        recordLog.setProductSn(productionPsOutContext.getProductSn());
        recordLog.setPartNo(productionPsOutContext.getPartNo());
        recordLog.setPartName(productionPsOutContext.getPartName());
        recordLog.setAssemblySn(context.getAssemblySn());
        recordLog.setAssemblyPartNo(context.getAssemblyPartNo());
        recordLog.setAssemblyPartName(context.getAssemblyPartName());
        recordLog.setOriginWeight(remainQty);
        recordLog.setDeductionWeight(context.getQty());
        recordLog.setRemainWeight(record.getNetWeight());
        ConvertBean.serviceModelInitialize(recordLog, username);
        rawSnWeightRecordLogRDao.insert(recordLog);
    }

    @Override
    public void insertOutPartNoWeightRecord(MesProductionPsOutContext productionPsOutContext, List<MesProductionAssemblyNosortContext> productionAssemblyNosortContextList, String organizeCode, String username) {
        MesPart mesPart = partService.getMesPartByPartNo(productionPsOutContext.getPartNo(), organizeCode);
        if (mesPart == null) {
            return;
        }
        double totalWeight = 0.0;
        for (MesProductionAssemblyNosortContext context : productionAssemblyNosortContextList) {
            if (Objects.equals(context.getMatchType(), MesExtEnumUtil.ASSEMBLY_MATCH_TYPE.MATCH_TYPE_90.getValue())) {
                totalWeight += context.getQty();
            }
        }

        MesRawSnWeightRecord record = new MesRawSnWeightRecord();
        record.setOrganizeCode(organizeCode);
        record.setBarCode(productionPsOutContext.getProductSn());
        record.setPartNo(mesPart.getPartNo());
        record.setPartName(mesPart.getPartName());
        record.setNetWeight(totalWeight);
        record.setNetUnit(mesPart.getWeightUom());
        record.setRawType(MesExtEnumUtil.RAW_MIX_DETAIL_TYPE.RAW_MIX_DETAIL_TYPE_10.getValue());
        ConvertBean.serviceModelInitialize(record, username);
        rawSnWeightRecordRDao.insert(record);
    }

    private double getRemainQty(String organizeCode, String partNo, String barcode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(barcode, "barCode", ddlPackBean);
        DdlPreparedPack.getOrderBy("createDatetime", CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), ddlPackBean);
        MesRawSnWeightRecord record = rawSnWeightRecordRDao.getByProperty(ddlPackBean);
        if (record != null) {
            return record.getNetWeight();
        }

        MesRawWeightCfg weightCfg = rawWeightCfgRDao.getByProperty(new String[]{"organizeCode", "isValid", "isDeleted", "partNo"},
                new Object[]{organizeCode, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue(), partNo});
        if (weightCfg == null) {
            return 0.0;
        }
        return weightCfg.getNetWeight();
    }
}
