package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesStationContainerSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesContainerPackageDetailContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 容器条码上料业务
 */
@Service
public class MesStationContainerSnExtService implements IMesStationContainerSnExtService {

    @Autowired
    private MesStationRepository stationRepository;

    @Autowired
    private MesContainerSnStationRepository containerSnStationRepository;

    @Autowired
    private MesContainerPackageRepository containerPackageRepository;

    @Autowired
    private MesContainerPackageDetailRepository containerPackageDetailRepository;

    @Autowired
    private MesContainerPackageDetailFeedingRepository containerPackageDetailFeedingRepository;

    //根据设备代码查询站点信息
    @Override
    public List<MesStation> getStationList(String organizeCode, String equipmentCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipmentCode)) return null;
        return stationRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.EQUIPMENT_CODE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), equipmentCode});
    }

    //获取站点关联的容器条码
    @Override
    public Map<String, List<MesContainerSnStation>> getContainerSnStationMap(String organizeCode, List<MesStation> stationList) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(stationList)) return null;
        List<MesContainerSnStation> containerSnStationList = getContainerSnStationList(organizeCode, stationList);
        return CollectionUtils.isEmpty(containerSnStationList) ? null :
                containerSnStationList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getContainerSn()))).collect(Collectors.groupingBy(o -> o.getContainerSn().toUpperCase()));
    }

    //获取站点关联的容器条码
    @Override
    public Map<String, List<MesContainerSnStation>> getContainerSnStationTopMap(String organizeCode, List<MesStation> stationList) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(stationList)) return null;
        List<MesContainerSnStation> containerSnStationList = getContainerSnStationList(organizeCode, stationList);
        return CollectionUtils.isEmpty(containerSnStationList) ? null :
                containerSnStationList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getTopContainerSn()))).collect(Collectors.groupingBy(o -> o.getTopContainerSn().toUpperCase()));
    }

    //获取站点关联的容器条码
    @Override
    public List<MesContainerSnStation> getContainerSnStationList(String organizeCode, List<MesStation> stationList) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(stationList)) return null;
        List<String> stations = stationList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getStation()))).map(MesStation::getStation).collect(Collectors.toList());
        return getContainerSnStationListByStation(organizeCode, stations);
    }

    //获取站点关联的容器条码
    @Override
    public List<MesContainerSnStation> getContainerSnStationListByStation(String organizeCode, List<String> stationList) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(stationList)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (stationList.size() == 1) DdlPreparedPack.getStringEqualPack(stationList.get(0), MesPcnExtConstWords.STATION, packBean);
        else DdlPreparedPack.getInPackList(stationList, MesPcnExtConstWords.STATION, packBean);
        return containerSnStationRepository.findByHqlWhere(packBean);
    }

    //根据容器条码查询容器条码与站点的对应关系
    @Override
    public MesContainerSnStation getContainerSnStation(String organizeCode, String containerSn) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(containerSn)) return null;
        return containerSnStationRepository.getByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.CONTAINER_SN},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), containerSn});
    }

    //根据容器条码查询容器条码与站点的对应关系
    @Override
    public MesContainerSnStation getContainerSnStationByStation(String organizeCode, String station) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(station)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(station, MesPcnExtConstWords.STATION, packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{MesPcnExtConstWords.SEQ}, packBean);
        return containerSnStationRepository.getByProperty(packBean);
    }

    //获取容器条码上料明细表信息【可扣减】【根据 主表seq,主表createDatetime,明细表createDatetime 正序】
    @Override
    public List<MesContainerPackageDetailContext> getContainerPackageDetailContext(String organizeCode, List<MesStation> stationList) {
        //获取站点关联的容器条码, key = containerSn
        Map<String, List<MesContainerSnStation>> containerSnStationMap = getContainerSnStationMap(organizeCode, stationList);
        if (CollectionUtils.isEmpty(containerSnStationMap)) return null;
        //获取容器条码上料主表信息【已关箱】
        List<MesContainerPackage> containerPackageList = getContainerPackageListByContainerSn(organizeCode,
                new ArrayList<>(containerSnStationMap.keySet()), MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_20.getValue());
        if (CollectionUtils.isEmpty(containerPackageList)) return null;
        //获取容器条码上料明细表信息
        List<MesContainerPackageDetail> containerPackageDetailList = getContainerPackageDetailList(organizeCode, containerPackageList);
        if (CollectionUtils.isEmpty(containerPackageDetailList)) return null;
        //站点根据名称分组
        Map<String, List<MesStation>> stationMap = stationList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getStation().toUpperCase()));
        //封装容器条码对应的【可扣减】的上料明细表信息
        List<MesContainerPackageDetailContext> containerSnStationContextList = new ArrayList<>();
        for (MesContainerPackageDetail containerPackageDetail : containerPackageDetailList) {
            if (null == containerPackageDetail) continue;
            MesContainerSnStation containerSnStation = containerSnStationMap.get(containerPackageDetail.getContainerSn().toUpperCase()).get(0);
            containerSnStationContextList.add(new MesContainerPackageDetailContext(containerPackageDetail).containerSnStation(containerSnStation).station(stationMap.get(containerSnStation.getStation().toUpperCase()).get(0)));
        }
        //根据 主表seq,主表createDatetime,明细表createDatetime 正序
        containerSnStationContextList = containerSnStationContextList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesContainerPackageDetailContext::getSeq)
                .thenComparing(MesContainerPackageDetailContext::getCreateDatetime2Package).thenComparing(MesContainerPackageDetailContext::getCreateDatetime)).collect(Collectors.toList());
        return containerSnStationContextList;
    }

    //获取容器条码上料主表信息
    @Override
    public List<MesContainerPackage> getContainerPackageList(String organizeCode, List<MesContainerSnStation> containerSnStationList, Integer packageStatus) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(containerSnStationList)) return null;
        List<String> containerSnList = containerSnStationList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getContainerSn()))).map(MesContainerSnStation::getContainerSn).collect(Collectors.toList());
        return getContainerPackageListByContainerSn(organizeCode, containerSnList, packageStatus);
    }

    //获取容器条码上料主表信息
    @Override
    public List<MesContainerPackage> getContainerPackageListByContainerSn(String organizeCode, List<String> containerSnList, Integer packageStatus) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(containerSnList)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (containerSnList.size() == 1)
            DdlPreparedPack.getStringEqualPack(containerSnList.get(0), MesPcnExtConstWords.CONTAINER_SN, packBean);
        else DdlPreparedPack.getInPackList(containerSnList, MesPcnExtConstWords.CONTAINER_SN, packBean);
        DdlPreparedPack.getNumEqualPack(packageStatus, MesPcnExtConstWords.PACKAGE_STATUS, packBean);
        return containerPackageRepository.findByHqlWhere(packBean);
    }

    //获取容器条码上料主表信息【进料】的一条数据
    @Override
    public MesContainerPackage getContainerPackageByContainerSn(String organizeCode, List<String> containerSnList, Integer packageStatus) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(containerSnList)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (containerSnList.size() == 1)
            DdlPreparedPack.getStringEqualPack(containerSnList.get(0), MesPcnExtConstWords.CONTAINER_SN, packBean);
        else DdlPreparedPack.getInPackList(containerSnList, MesPcnExtConstWords.CONTAINER_SN, packBean);
        DdlPreparedPack.getNumEqualPack(packageStatus, MesPcnExtConstWords.PACKAGE_STATUS, packBean);
        return containerPackageRepository.getByProperty(packBean);
    }

    @Override
    public MesContainerPackage getContainerPackageById(String organizeCode, Long id) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(id)) return null;
        return containerPackageRepository.getByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.ID},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), id});
    }

    //获取容器条码上料明细表信息【可扣减】
    @Override
    public List<MesContainerPackageDetail> getContainerPackageDetailList(String organizeCode, List<MesContainerPackage> containerPackageList) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(containerPackageList)) return null;
        List<Long> idList = containerPackageList.stream().filter(o -> null != o).map(MesContainerPackage::getId).collect(Collectors.toList());
        return getContainerPackageDetailListByPid(organizeCode, idList);
    }

    //获取容器条码上料明细表信息【可扣减】
    @Override
    public List<MesContainerPackageDetail> getContainerPackageDetailListByPid(String organizeCode, List<Long> pidList) {
        return getContainerPackageDetailListByPid(organizeCode, pidList, MesExtEnumUtil.CONTAINER_BARCODE_STATUS.STATUS_10.getValue());
    }

    //获取容器条码上料明细表信息
    @Override
    public List<MesContainerPackageDetail> getContainerPackageDetailListByPid(String organizeCode, List<Long> pidList, Integer snStatus) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(pidList)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (pidList.size() == 1) DdlPreparedPack.getNumEqualPack(pidList.get(0), MesPcnExtConstWords.PID, packBean);
        else DdlPreparedPack.getInPackList(pidList, MesPcnExtConstWords.PID, packBean);
        DdlPreparedPack.getNumEqualPack(snStatus, MesPcnExtConstWords.SN_STATUS, packBean);
        return containerPackageDetailRepository.findByHqlWhere(packBean);
    }

    //获取容器条码上料明细表信息
    @Override
    public List<MesContainerPackageDetail> getContainerPackageDetailListByPid(String organizeCode, Long pid) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(pid)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(pid, MesPcnExtConstWords.PID, packBean);
        return containerPackageDetailRepository.findByHqlWhere(packBean);
    }

    //根据单件条码查询上料明细表信息
    @Override
    public MesContainerPackageDetail getContainerPackageDetailByBarCode(String organizeCode, String barCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(barCode)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(barCode, MesPcnExtConstWords.BAR_CODE, packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{MesPcnExtConstWords.CREATE_DATE_TIME}, packBean);
        return containerPackageDetailRepository.getByProperty(packBean);
    }

    //获取容器条码的一条上料明细表信息【可扣减】
    @Override
    public Boolean checkIsExistContainerPackageDetail(String organizeCode, Long id) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(id)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(id, MesPcnExtConstWords.PID, packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.CONTAINER_BARCODE_STATUS.STATUS_10.getValue(), MesPcnExtConstWords.SN_STATUS, packBean);
        return containerPackageDetailRepository.isExitByHql(packBean);
    }

    //写入扣料记录
    @Override
    public void insertContainerPackageDetailFeeding(StationRequestBean reqBean, MesContainerPackageDetailContext containerPackageDetailContext, MesProductionPsOutContext productionPsOutContext) {
        MesContainerPackageDetailFeeding containerPackageDetailFeeding = new MesContainerPackageDetailFeeding();
        BeanUtils.copyProperties(containerPackageDetailContext, containerPackageDetailFeeding, MesPcnExtConstWords.BASE_BEAN_FIELDS);
        containerPackageDetailFeeding.setQty(containerPackageDetailContext.getDeductionQty());
        containerPackageDetailFeeding.setSourceId(containerPackageDetailContext.getId());
        if (null != productionPsOutContext) {
            containerPackageDetailFeeding.setProductSn(productionPsOutContext.getProductSn());
            containerPackageDetailFeeding.setProductSnId(productionPsOutContext.getId());
        }
        ConvertBean.serviceModelInitialize(containerPackageDetailFeeding, reqBean.getUserInfo());
        containerPackageDetailFeedingRepository.insert(containerPackageDetailFeeding);
    }

    @Override
    public void updateContainerPackageDetail(String organizeCode, String userInfo, Integer snStatus, Long... ids) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(userInfo) || StringUtils.isEmpty(snStatus) || null == ids) return;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (ids.length == 1) DdlPreparedPack.getNumEqualPack(ids[0], MesPcnExtConstWords.ID, packBean);
        else DdlPreparedPack.getInPackArray(ids, MesPcnExtConstWords.ID, packBean);
        containerPackageDetailRepository.updateByProperties(
                new String[]{MesPcnExtConstWords.SN_STATUS, MesPcnExtConstWords.MODIFY_USER, MesPcnExtConstWords.MODIFY_DATE_TIME, MesPcnExtConstWords.SYSTEM_SYNC_STATUS, MesPcnExtConstWords.SYSTEM_SYNC_DATE_TIME},
                new Object[]{snStatus, userInfo, TimeTool.getNowTime(true), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), MesPcnExtConstWords.EMPTY},
                packBean);
    }

    @Override
    public void insertContainerSnStationList(String organizeCode, String userInfo, String station, List<MesContainerPackage> containerPackageList, String topContainerSn) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(station) || CollectionUtils.isEmpty(containerPackageList)) return;
        if (StringUtils.isEmpty(topContainerSn)) topContainerSn = containerPackageList.get(0).getContainerSn();
        MesContainerSnStation containerSnStation = getContainerSnStationByStation(organizeCode, station);
        Integer seq = (null != containerSnStation && !StringUtils.isEmpty(containerSnStation.getSeq())) ? containerSnStation.getSeq() + 1 : MesPcnExtConstWords.ONE;
        for (MesContainerPackage containerPackage : containerPackageList) {
            if (null == containerPackage) continue;
            MesContainerSnStation css = new MesContainerSnStation();
            css.setStation(station);
            css.setContainerSn(containerPackage.getContainerSn());
            css.setTopContainerSn(topContainerSn);
            css.setSeq(seq);
            css.setLotNo(TimeTool.getToday());
            css.setOrganizeCode(organizeCode);
            ConvertBean.serviceModelInitialize(css, userInfo);
            containerSnStationRepository.insert(css);
        }
    }

}
