package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesProdShiftRecordService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesStationContainerSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.method.stationpm.IMesStationMatchPsProcessMethodStrategyStepService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesReentrantLockUtil;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.*;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.actor.shipping.dispatch.IFsmCommonService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.IStepService;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description : 匹配原料站点工步
 * @Author : wangjie
 **/
@Slf4j
@Service("mesStationMatchProductSnStepService")
public class MesStationMatchProductSnStepService extends BaseStepService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private MesProductSnCheckStepService productSnCheckStepService;

    @Autowired
    private MesAssemblyShowNosortStepService assemblyShowNosortStepService;

    @Autowired
    private MesProductSnGenerateStepService productSnGenerateStepService;

    @Autowired
    private IMesStationContainerSnExtService stationContainerSnExtService;

    @Autowired
    private IMesProdShiftRecordService prodShiftRecordService;

    @Autowired
    private IFsmCommonService fsmCommonService;

    @Override
    public StepResult init(StationRequestBean reqBean) {

        StepResult stepResult = StepResult.getSuccessComplete();

        String endlessLoopReadTimes = fsmCommonService.handleFsmWcpcMapDataForDoScanThenBackValue(reqBean, MesPcnExtConstWords.ENDLESS_LOOP_READ_TIMES);
        if (StringUtils.isEmpty(endlessLoopReadTimes)) endlessLoopReadTimes = MesPcnExtConstWords.ENDLESS_LOOP_READ_TIMES_DEFAULT;
        if (productionDispatchContextStepService.dispatchOverEndlessLoopReadTimes(reqBean, endlessLoopReadTimes)) {
            stepThreadSleepAndSendTaskCompleteAndThrowEx(reqBean, new StationResultBean().isWs(false).writeDbLog().checkRepeat(),
                    stepResult.isCompleted(false).msg(String.format("当前未能执行完成匹配原料站点超过[%s]次!", endlessLoopReadTimes)),
                    MesPcnEnumUtil.STATION_BUSI_TYPE.RUNNING_INFO, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT, getStepParams(reqBean), MesPcnExtConstWords.READ_FAILURE_SLEEP, MesPcnExtConstWords.READ_FAILURE_SLEEP_DEFAULT_TIME);
        }

        return stepResult;

    }

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        //获取工位当前设备信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.dispatchCurCellEquipment(reqBean);

        //配置错误 抛出异常
        if (!productionProcessContext.getSuccess()) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), productionProcessContext.getMessage());

        //获取生产线的当前班组班次信息
        MesProdShiftContext prodShiftContext = prodShiftRecordService.getMesProdShiftKvBean(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());
        if (null == prodShiftContext) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "当前不存在生产开班记录,请重新开班!");

        //处理设备站点【原料站点&&混料站点&&成品站点&&可疑站点】
        List<MesStation> stationList = productionProcessContextStepService.dispatchEquipmentStationContext(reqBean, productionProcessContext.getCurCellEquip().getEquipmentCode(),
                Stream.of(MesExtEnumUtil.STATION_TYPE.STATION_TYPE_10.getValue(), MesExtEnumUtil.STATION_TYPE.STATION_TYPE_40.getValue(),
                        MesExtEnumUtil.STATION_TYPE.STATION_TYPE_30.getValue(), MesExtEnumUtil.STATION_TYPE.STATION_TYPE_50.getValue()).collect(Collectors.toList()));

        //剔除没有加工模式的站点数据
        List<MesStation> filterList = stationList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getProcessMethod()) && !StringUtils.isEmpty(MesExtEnumUtil.STATION_PROCESS_METHOD.valueOfDescription(o.getProcessMethod())))).collect(Collectors.toList());
        //原料站点
        List<MesStation> stationList10 = productionProcessContextStepService.dispatchEquipmentStationContext(filterList, Stream.of(MesExtEnumUtil.STATION_TYPE.STATION_TYPE_10.getValue()).collect(Collectors.toList()));
        //混料站点
        List<MesStation> stationList40 = productionProcessContextStepService.dispatchEquipmentStationContext(filterList, Stream.of(MesExtEnumUtil.STATION_TYPE.STATION_TYPE_40.getValue()).collect(Collectors.toList()));
        //搜集设备站点 【进料】【原料站点&&混料站点】
        List<MesStation> stationList2Ps = filterStationList2Ps(reqBean, resultBean, stationList10, stationList40);

        //验证【成品站点】是否存在【未关箱】的容器条码
        if (!beforeCheckIsExistContainerSn(reqBean, resultBean, stepResult,
                productionProcessContextStepService.dispatchEquipmentStationContext(stationList, Stream.of(MesExtEnumUtil.STATION_TYPE.STATION_TYPE_30.getValue()).collect(Collectors.toList())),
                MesExtEnumUtil.STATION_TYPE.STATION_TYPE_30.getDescription(), MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_10).isCompleted()) return stepResult;

        //验证【可疑站点】是否存在【未关箱】的容器条码
        if (!beforeCheckIsExistContainerSn(reqBean, resultBean, stepResult,
                productionProcessContextStepService.dispatchEquipmentStationContext(stationList, Stream.of(MesExtEnumUtil.STATION_TYPE.STATION_TYPE_50.getValue()).collect(Collectors.toList())),
                MesExtEnumUtil.STATION_TYPE.STATION_TYPE_50.getDescription(), MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_10).isCompleted()) return stepResult;

        //验证【混料站点】是否存在容器条码
        if (!beforeCheckIsExistContainerSn(reqBean, resultBean, stepResult,
                productionProcessContextStepService.dispatchEquipmentStationContext(stationList, Stream.of(MesExtEnumUtil.STATION_TYPE.STATION_TYPE_40.getValue()).collect(Collectors.toList())),
                MesExtEnumUtil.STATION_TYPE.STATION_TYPE_40.getDescription(), CollectionUtils.isEmpty(stationList10) ? MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_20 : null).isCompleted()) return stepResult;

        //保存站点用于绑定进料容器条码
        productionDispatchContextStepService.dispatchMatchStationProductSnContext(reqBean, stationList2Ps);

        //加工模式策略
        MesStation station = stationList2Ps.get(0);
        IMesStationMatchPsProcessMethodStrategyStepService strategyService = (IMesStationMatchPsProcessMethodStrategyStepService) SpringContextsUtil.getBean(MesExtEnumUtil.STATION_PROCESS_METHOD.valueOfStrategyClass(station.getProcessMethod()));

        //【原料站点】获取容器条码上料明细表信息
        MesContainerPackageDetailStationContext containerPackageDetailStationContext = strategyService.getContainerPackageDetailStationContext(reqBean.getOrganizeCode(), stationList2Ps);
        if (null == containerPackageDetailStationContext) {
            return stepNonCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult.nextTriggerEvent(MesPcnExtConstWords.NEXT_TRIGGER_EVENT_FEEDING), "匹配原料站点时验证[原料]站点当前已无进料信息!");
        }

        //从上下文中取出生产线对象
        MesWorkCenter workCenter = productionProcessContext.getWorkCenter();
        //从上下文中取出工位当前要使用的设备
        MesCellEquipContext cellEquipContext = productionProcessContext.getCurCellEquip();

        //获取上下文产出零件信息
        List<MesProductionPartContext> productionPartContextList = productionDispatchContextStepService.getProductionPartContext(reqBean);
        //剔除空腔
        List<MesProductionPartContext> productionPartContextList2UnFinish = CollectionUtils.isEmpty(productionPartContextList) ? null : productionPartContextList.stream().filter(o -> (null != o && o.getIsFinishCode().compareTo(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) == 0)).collect(Collectors.toList());

        //进料主条码数据信息
        List<MesProductionPsInContext> productionPsInContextList = new ArrayList<>();
        //加工规则数据信息集合
        List<MesProdRuleContext> prodRuleContextList = new ArrayList<>();

        //获取工步参数
        Optional<Map<String, MesProdRouteOptParam>> stepParamMap = getStepParams(reqBean);
        //是否验证工艺路线配置
        String isCheckCraft = (null != stepParamMap && stepParamMap.isPresent() && stepParamMap.get().containsKey(MesPcnExtConstWords.IS_CHECK_CRAFT_CFG)) ? stepParamMap.get().get(MesPcnExtConstWords.IS_CHECK_CRAFT_CFG).getParamValue() : null;

        //需要进行LOCK的站点
        List<String> stationNameList = containerPackageDetailStationContext.getStationList().stream().filter(o -> null != o).map(o -> o.getStation().toUpperCase()).collect(Collectors.toList());

        // 结尾需要缓存锁定的数据
        List<MesContainerPackageDetailContext> needCachedList = null;

        List<ReentrantLock> acquiredLocks = new ArrayList<>();
        try {
            if (!MesReentrantLockUtil.tryLock(acquiredLocks, stationNameList, MesPcnExtConstWords.MATCH_STATION_PRODUCT_SN_CONTEXT + MesPcnExtConstWords.COLON, 3000L)) {
                return stepDynamicsCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog().checkRepeat(), stepResult, false,
                        MesPcnEnumUtil.STATION_BUSI_TYPE.RUNNING_INFO, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT, "匹配原料站点时LOCK失败,即将重试!");
            }

            //循环到全部验证成功或根据场景验证失败结束
            for (int i = 0; i >= 0; i ++) {

                stepResult = StepResult.getSuccessComplete();

                //【原料站点】获取进料主条码信息
                List<MesContainerPackageDetailContext> containerPackageDetailContextList = strategyService.getContainerPackageDetailContext(reqBean, containerPackageDetailStationContext, cellEquipContext.getCavity(), !CollectionUtils.isEmpty(productionPartContextList2UnFinish));

                //验证是否满足腔数【前置验证】
                if (!strategyService.checkIsMatchCavity(getListSize(containerPackageDetailContextList), getListSize(prodRuleContextList), getListSize(productionPartContextList2UnFinish), cellEquipContext.getCavity(), true)) {
                    return stepNonCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult.nextTriggerEvent(MesPcnExtConstWords.NEXT_TRIGGER_EVENT_FEEDING), "匹配原料站点时验证[原料]站点当前进料不足!");
                }

                //封装主条码信息上下文
                List<MesEquipVariableCollectContext> equipVariableCollectContextList = new ArrayList<>();
                containerPackageDetailContextList.stream().filter(o -> null != o).forEach(o ->
                        equipVariableCollectContextList.add(new MesEquipVariableCollectContext(reqBean.getOrganizeCode(), o.getBarCode(), MesExtEnumUtil.CELL_MESSAGE_SOURCE.READ.getValue()).qty(o.getQty()).contextInfo(o.getId().toString())));

                //处理条码验证逻辑 【条码验证失败直接抛出异常】
                productionPsInContextList = productSnCheckStepService.checkProduceSnValid(reqBean, resultBean, stepResult, productionProcessContext, equipVariableCollectContextList, productionPsInContextList);
                if (!stepResult.isCompleted()) stepSendGuideAndThrowEx(reqBean, new StationResultBean().writeDbLog(), "主条码验证失败!请人工处理!");

                //显示装配件扫描项工步【非排序】【业务处理】
                assemblyShowNosortStepService.doHandleProdRuleContext(reqBean, new StationResultBean().isIgnore(true), stepResult, workCenter, productionProcessContext, cellEquipContext, prodRuleContextList, productionPartContextList, productionPsInContextList);

                //前道工艺防错工步【非排序】
                if (stepResult.isCompleted() && !StringUtils.isEmpty(isCheckCraft) && !(((IStepService) SpringContextsUtil.getBean("mesProdCraftRouteCheckNosortStepService")).executeInState(reqBean)).isCompleted()) {
                    stepSendGuideAndThrowEx(reqBean, new StationResultBean().writeDbLog(), "主条码前道工艺防错验证失败!请人工处理!");
                }

                //验证是否满足腔数【后置验证】
                Boolean result = strategyService.checkIsMatchCavity(getListSize(containerPackageDetailContextList), getListSize(prodRuleContextList), getListSize(productionPartContextList2UnFinish), cellEquipContext.getCavity(), false);
                if (null != result && !result) {
                    stepNonCompleteAndSendMsg(reqBean, resultBean.writeDbLog(), stepResult, stepResult.getMsg());
                    stepSendGuideAndThrowEx(reqBean, new StationResultBean().writeDbLog(), "加工规则匹配失败!请人工处理!");
                }

                if (stepResult.isCompleted()) {
                    if (CollectionUtils.isEmpty(needCachedList)) needCachedList = new ArrayList<>();
                    needCachedList.addAll(containerPackageDetailContextList);
                }

                if (null != result && result) break;

                //result为null需要继续循环处理
                containerPackageDetailContextList.get(0).matchResult(stepResult.isCompleted() ? CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValueStr() : CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValueStr());

            }

            //锁定当前已被验证了的主条码或容器条码
            strategyService.dispatchContainerPackageDetailPsLocked(reqBean, needCachedList);

        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            return stepDynamicsCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog().checkRepeat(), stepResult, false,
                    MesPcnEnumUtil.STATION_BUSI_TYPE.RUNNING_INFO, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT, "匹配原料站点异常,即将重试!");
        } finally {

            MesReentrantLockUtil.unLockAll(acquiredLocks);

            if (!stepResult.isCompleted()) {
                if (!CollectionUtils.isEmpty(productionPsInContextList)) productionDispatchContextStepService.removeProductionPsInContext(reqBean);
                if (!CollectionUtils.isEmpty(prodRuleContextList)) productionDispatchContextStepService.removeProdRuleDataContext(reqBean);
            }

        }

        //显示装配件扫描项
        assemblyShowNosortStepService.showProductionAssembly(reqBean, resultBean, workCenter, prodRuleContextList);

        //生产零件条码业务处理
        productSnGenerateStepService.doHandleProductSnGenerate(reqBean, resultBean, stepResult, productionProcessContext, productionPartContextList, productionPsInContextList, prodRuleContextList, prodShiftContext);

        //缓存已被验证了的容器主条码信息
        productionDispatchContextStepService.dispatchContainerPackageDetailPs(reqBean, needCachedList);

        return stepSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "匹配原料站点验证成功!", MesPcnEnumUtil.PROMPT_SOUND.SUCCESS.getValue());

    }

    //获取集合的个数
    private Integer getListSize(List list) { return CollectionUtils.isEmpty(list) ? MesPcnExtConstWords.ZERO : list.size(); }

    //搜集设备站点 【进料】【原料站点&&混料站点】
    private List<MesStation> filterStationList2Ps(StationRequestBean reqBean, StationResultBean resultBean, List<MesStation> stationList10, List<MesStation> stationList40) {

        //原料站点根据加工模式分组
        Map<Integer, List<MesStation>> processMethodMap10 = CollectionUtils.isEmpty(stationList10) ? null : stationList10.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesStation::getProcessMethod));
        //混料站点根据加工模式分组
        Map<Integer, List<MesStation>> processMethodMap40 = CollectionUtils.isEmpty(stationList40) ? null : stationList40.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesStation::getProcessMethod));

        if (CollectionUtils.isEmpty(processMethodMap10) && CollectionUtils.isEmpty(processMethodMap40)) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), String.format("匹配原料站点时验证设备未关联有效的[%s]站点!",
                new StringJoiner(MesPcnExtConstWords.SLANT_R).add(MesExtEnumUtil.STATION_TYPE.STATION_TYPE_10.getDescription()).add(MesExtEnumUtil.STATION_TYPE.STATION_TYPE_40.getDescription()).toString()));

        //验证加工模式, 原料站点只能维护一种加工模式
        if (!CollectionUtils.isEmpty(processMethodMap10) && processMethodMap10.size() != 1) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "匹配原料站点时验证设备关联的原料站点的加工模式配置错误!");
        //验证加工模式, 原料站点不存在的时候, 混料站点只能维护一种加工模式
        if (CollectionUtils.isEmpty(processMethodMap10) && processMethodMap40.size() != 1) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "匹配原料站点时验证设备关联的混料站点的加工模式配置错误!");

        //存在原料站点时,合并跟原料站点相同加工模式的混料站点
        if (!CollectionUtils.isEmpty(stationList10)) {
            stationList40 = CollectionUtils.isEmpty(stationList40) ? null : stationList40.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getProcessMethod()) && o.getProcessMethod().compareTo(stationList10.get(0).getProcessMethod()) == 0)).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(stationList40)) stationList10.addAll(stationList40);
            return stationList10;
        }

        //不存在原料站点时, 返回混料站点
        return stationList40;

    }

    //前置验证不同类型的站点是否至少存在一个保底的容器条码用于存放产成品
    private StepResult beforeCheckIsExistContainerSn(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult,
                                              List<MesStation> stationList, String suffix, MesExtEnumUtil.CONTAINER_PACKAGE_STATUS containerPackageStatus) {
        if (CollectionUtils.isEmpty(stationList)) return stepResult;

        //获取站点关联的容器条码【进料】
        Map<String, List<MesContainerSnStation>> containerSnStationMap = stationContainerSnExtService.getContainerSnStationTopMap(reqBean.getOrganizeCode(), stationList);
        List<String> containerSnList = CollectionUtils.isEmpty(containerSnStationMap) ? null : new ArrayList<>(containerSnStationMap.keySet());

        //获取容器条码上料主表信息【进料】的一条数据
        MesContainerPackage containerPackage = stationContainerSnExtService.getContainerPackageByContainerSn(
                reqBean.getOrganizeCode(), containerSnList, null == containerPackageStatus ? null : containerPackageStatus.getValue());
        if (null != containerPackage) return stepResult;

        //成品|可疑场景下,如果上面查询的containerPackage不存在, 需要判断当前所有容器条码是否全部是已封箱的, 如果是则验证不通过 【不存在已关箱状态的数据即默认是未关箱的】
        if (null != containerPackageStatus && MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_10.getValue() == containerPackageStatus.getValue()) {
            List<MesContainerPackage> containerPackageList = stationContainerSnExtService.getContainerPackageListByContainerSn(
                    reqBean.getOrganizeCode(), containerSnList, MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_20.getValue());
            List<String> containerSnListDistinct = CollectionUtils.isEmpty(containerPackageList) ? null :
                    (containerPackageList.stream().filter(o -> null != o).map(MesContainerPackage::getContainerSn).collect(Collectors.toList()))
                            .stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(containerSnList) && (CollectionUtils.isEmpty(containerSnListDistinct) || containerSnListDistinct.size() != containerSnList.size())) return stepResult;
        }

        //保存站点用于绑定进料容器条码
        productionDispatchContextStepService.dispatchMatchStationProductSnContext(reqBean, stationList);
        return stepNonCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult.nextTriggerEvent(MesPcnExtConstWords.NEXT_TRIGGER_EVENT_FEEDING),
                String.format("匹配原料站点时验证[%s]站点未绑定%s容器条码!", suffix, null == containerPackageStatus ? MesPcnExtConstWords.EMPTY : String.format("[%s]状态的", containerPackageStatus.getDescription())));

    }

}
