package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.method.stationpm;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesContainerPackageDetailContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesContainerPackageDetailStationContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.mes.bean.MesStation;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description : 加工模式 - 整箱
 * @Author : wangjie
 **/
@Slf4j
@Service("mesStationMatchPsProcessMethodContainerStepService")
public class MesStationMatchPsProcessMethodContainerStepService extends BaseStepService implements IMesStationMatchPsProcessMethodStrategyStepService {

    @Autowired
    private IMesStationMatchPsProcessMethodStrategyStepService defaultDecoratorService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    //【原料站点】获取容器条码上料明细表信息
    @Override
    public MesContainerPackageDetailStationContext getContainerPackageDetailStationContext(String organizeCode, List<MesStation> stationList) {
        return getContainerPackageDetailStationContext(defaultDecoratorService, organizeCode, stationList);
    }

    //【原料站点】获取容器条码上料明细表信息
    @Override
    public MesContainerPackageDetailStationContext getContainerPackageDetailStationContext(IMesStationMatchPsProcessMethodStrategyStepService decoratorService, String organizeCode, List<MesStation> stationList) {
        MesContainerPackageDetailStationContext decoratorContext = decoratorService.getContainerPackageDetailStationContext(organizeCode, stationList);
        if (null == decoratorContext) return decoratorContext;
        return new MesContainerPackageDetailStationContext().addStationList(decoratorContext.getStationList()).addStationContainerSnDetailList(decoratorContext);
    }

    //【原料站点】获取进料主条码信息
    @Override
    public List<MesContainerPackageDetailContext> getContainerPackageDetailContext(StationRequestBean reqBean, MesContainerPackageDetailStationContext context, Integer cavity, Boolean isExistProductionPart) {
        if (null == context) return null;
        List<MesContainerPackageDetailStationContext> containerPackageDetailStationContextList = context.getStationContainerSnDetailList();
        for (MesContainerPackageDetailStationContext containerPackageDetailStationContext : containerPackageDetailStationContextList) {
            if (null == containerPackageDetailStationContext) continue;
            LinkedHashMap<String, List<MesContainerPackageDetailContext>> containerSnDataMap = containerPackageDetailStationContext.getTopContainerSnDetailMap2Sort();
            for (Map.Entry<String, List<MesContainerPackageDetailContext>> entry : containerSnDataMap.entrySet()) {
                if (null == entry || !StringUtils.isEmpty(entry.getValue().get(0).getIsUsed())) continue;
                //验证是否被使用
                if (productionCustomContextStepService.checkContainerPackageDetailPsLockedIsUsed(reqBean, MesPcnExtConstWords.CONTAINER_SN, entry.getKey())) {
                    //标记被使用
                    entry.getValue().get(0).isUsed();
                    continue;
                }
                //返回整箱
                return entry.getValue();
            }
        }
        return null;
    }

    //【原料站点】验证进料主条码个数是否满足腔数
    // count1=进料主条码个数   count2=加工规则对象个数   count3=产成零件个数[腔数]  cavity=设备腔数
    @Override
    public Boolean checkIsMatchCavity(Integer count1, Integer count2, Integer count3, Integer cavity, Boolean isBeforeCheck) {
        //前置验证
        if (isBeforeCheck) {
            //不存在产成,按箱加工则不验证腔数,箱内条数即腔数
            if (count3.compareTo(MesPcnExtConstWords.ZERO) == 0) return true;
            //存在产成, 验证进料是否满足产成腔数
            return count1.compareTo(count3) == 0;
        }

        //后置验证  为true或者false 均退出循环
        return count1.compareTo(count2) == 0;
    }

    //【原料站点】锁定进料主条码信息
    @Override
    public void dispatchContainerPackageDetailPsLocked(StationRequestBean reqBean, List<MesContainerPackageDetailContext> containerPackageDetailContextList) {
        productionCustomContextStepService.dispatchContainerPackageDetailPsLocked(reqBean, MesPcnExtConstWords.CONTAINER_SN, containerPackageDetailContextList.get(0).getTopContainerSnKey());
    }

    //【原料站点】解锁进料主条码信息
    @Override
    public void removeContainerPackageDetailPsLocked(StationRequestBean reqBean, List<MesContainerPackageDetailContext> containerPackageDetailContextList) {
        productionCustomContextStepService.removeContainerPackageDetailPsLocked(reqBean, MesPcnExtConstWords.CONTAINER_SN, containerPackageDetailContextList.get(0).getTopContainerSnKey());
    }

}
