package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.method.stationpm;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesStationContainerSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.StringEqualsUtil;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesContainerPackageDetailContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesContainerPackageDetailStationContext;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerPackage;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerPackageDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerSnStation;
import cn.estsh.i3plus.pojo.mes.bean.MesStation;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 加工模式 - Decorator
 * @Author : wangjie
 **/
@Slf4j
@Primary
@Service("mesStationMatchPsProcessMethodDecoratorStepService")
public class MesStationMatchPsProcessMethodDecoratorStepService extends BaseStepService implements IMesStationMatchPsProcessMethodStrategyStepService {

    @Autowired
    private IMesStationContainerSnExtService stationContainerSnExtService;

    @Override
    public MesContainerPackageDetailStationContext getContainerPackageDetailStationContext(String organizeCode, List<MesStation> stationList) {

        //获取站点关联的容器条码
        List<MesContainerSnStation> containerSnStationList = stationContainerSnExtService.getContainerSnStationList(organizeCode, stationList);
        containerSnStationList = CollectionUtils.isEmpty(containerSnStationList) ? null :
                containerSnStationList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getContainerSn()) && !StringUtils.isEmpty(o.getTopContainerSn()))).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(containerSnStationList)) return null;

        //容器条码根据TOP容器条码分组
        Map<String, List<MesContainerSnStation>> topContainerSnMap = CollectionUtils.isEmpty(containerSnStationList) ? null :
                containerSnStationList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getTopContainerSn().toUpperCase()));
        //容器条码根据容器条码分组
        Map<String, List<MesContainerSnStation>> containerSnMap = CollectionUtils.isEmpty(containerSnStationList) ? null :
                containerSnStationList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getContainerSn().toUpperCase()));

        //容器条码根据创建时间正序 【暂不考虑seq, 因为站点没有顺序, 没法比较多站点的情况, 一个站点的情况下, 创建时间正序正常情况下等同于seq正序】
        containerSnStationList = containerSnStationList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesContainerSnStation::getCreateDatetime)).collect(Collectors.toList());
        //根据TOP去重
        List<MesContainerSnStation> containerSnStationList2TopDistinct = containerSnStationList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getTopContainerSn())))
                .distinct().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(o -> o.getTopContainerSn().toUpperCase()))), ArrayList::new));
        //搜集料架车容器条码
        List<String> topContainerSnList = containerSnStationList2TopDistinct.stream().filter(o -> (null != o && !StringEqualsUtil.toUpperCaseEquals(o.getTopContainerSn(), o.getContainerSn()))).map(o -> o.getTopContainerSn().toUpperCase()).collect(Collectors.toList());

        //料桶容器条码
        List<String> containerSnList = new ArrayList<>(containerSnMap.keySet());
        //最外层料架车与料桶容器条码
        List<String> allContainerSnList = new ArrayList<>(containerSnList);
        //合并
        if (!CollectionUtils.isEmpty(topContainerSnList)) allContainerSnList.addAll(topContainerSnList);
        //去重
        allContainerSnList = allContainerSnList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());

        //获取合并的容器条码上料主表信息【已关箱】
        List<MesContainerPackage> containerPackageList = stationContainerSnExtService.getContainerPackageListByContainerSn(organizeCode, allContainerSnList, MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_20.getValue());
        //料桶对应的上料主表数据
        List<MesContainerPackage> containerPackageList2Detail = CollectionUtils.isEmpty(containerPackageList) ? null :
                containerPackageList.stream().filter(o -> (null != o && containerSnList.contains(o.getContainerSn().toUpperCase()))).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(containerPackageList) || CollectionUtils.isEmpty(containerPackageList2Detail)) return null;

        //获取料桶对应的上料明细表信息【可扣减】
        List<MesContainerPackageDetail> containerPackageDetailList = stationContainerSnExtService.getContainerPackageDetailList(organizeCode, containerPackageList2Detail);
        if (CollectionUtils.isEmpty(containerPackageDetailList)) return null;

        //已关箱的容器条码信息根据条码分组
        Map<String, List<MesContainerPackage>> containerPackageMap = containerPackageList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getContainerSn().toUpperCase()));

        //站点根据名称分组
        Map<String, List<MesStation>> stationMap = stationList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getStation().toUpperCase()));
        Map<String, MesStation> stationMap2Back = new HashMap<>();
        //封装容器条码对应的【可扣减】的上料明细表信息
        List<MesContainerPackageDetailContext> containerSnStationContextList = new ArrayList<>();
        for (MesContainerPackageDetail containerPackageDetail : containerPackageDetailList) {
            if (null == containerPackageDetail) continue;
            MesContainerSnStation containerSnStation = containerSnMap.get(containerPackageDetail.getContainerSn().toUpperCase()).get(0);
            containerSnStationContextList.add(
                    new MesContainerPackageDetailContext(containerPackageDetail)
                            .containerSnStation(containerSnStation, containerPackageMap.get(containerSnStation.getTopContainerSn().toUpperCase()).get(0).getId())
                            .station(stationMap.get(containerSnStation.getStation().toUpperCase()).get(0))
            );
            if (!stationMap2Back.containsKey(containerSnStation.getStation().toUpperCase())) stationMap2Back.put(containerSnStation.getStation().toUpperCase(), stationMap.get(containerSnStation.getStation().toUpperCase()).get(0));
        }

        //上料明细数据根据TOP容器条码分组
        Map<String, List<MesContainerPackageDetailContext>> topContainerSnDetailMap = containerSnStationContextList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getTopContainerSn().toUpperCase()));

        //已关箱的料架车容器条码
        List<String> topContainerSnList2Colsed = (CollectionUtils.isEmpty(containerPackageList) || CollectionUtils.isEmpty(topContainerSnList)) ? null :
                containerPackageList.stream().filter(o -> (null != o && topContainerSnList.contains(o.getContainerSn().toUpperCase()))).map(o -> o.getContainerSn().toUpperCase()).collect(Collectors.toList());

        LinkedHashMap<String, List<MesContainerPackageDetailContext>> topContainerSnDetailMap2Sort = new LinkedHashMap<>();
        //遍历 根据创建时间正序,TOP去重后的容器条码
        for (MesContainerSnStation containerSnStation : containerSnStationList2TopDistinct) {
            //跳过被扣减完的容器条码
            if (null == containerSnStation || !topContainerSnDetailMap.containsKey(containerSnStation.getTopContainerSn().toUpperCase())) continue;
            //跳过未封箱的料架车
            if (!StringEqualsUtil.toUpperCaseEquals(containerSnStation.getTopContainerSn(), containerSnStation.getContainerSn()) && !topContainerSnList2Colsed.contains(containerSnStation.getTopContainerSn().toUpperCase())) continue;
            //将明细数据按照主表createDatetime,明细表createDatetime 正序 后, 存入有序集合内
            List<MesContainerPackageDetailContext> detailList2Sort = topContainerSnDetailMap.get(containerSnStation.getTopContainerSn().toUpperCase()).stream().filter(o -> null != o)
                    .sorted(Comparator.comparing(MesContainerPackageDetailContext::getCreateDatetime2Package).thenComparing(MesContainerPackageDetailContext::getCreateDatetime)).collect(Collectors.toList());
            topContainerSnDetailMap2Sort.put(detailList2Sort.get(0).getTopContainerSnKey(), detailList2Sort);
        }

        return new MesContainerPackageDetailStationContext(stationMap2Back.values().stream().collect(Collectors.toList()), topContainerSnDetailMap2Sort);

    }

}
